/**
 * Copyright 2009-2012, Descartes Research Group, SDQ, IPD, KIT
 */
package tools.descartes.dml.mm.applicationlevel.system.provider;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.ViewerNotification;

import tools.descartes.dml.mm.applicationlevel.system.Hinge;
import tools.descartes.dml.mm.applicationlevel.system.Side;
import tools.descartes.dml.mm.applicationlevel.system.SystemFactory;
import tools.descartes.dml.mm.applicationlevel.system.SystemPackage;
import tools.descartes.dml.mm.applicationlevel.system.WorkloadIntensity;

/**
 * This is the item provider adapter for a
 * {@link tools.descartes.dml.mm.applicationlevel.system.Hinge} object. <!--
 * begin-user-doc --> <!-- end-user-doc -->
 * 
 * @generated
 */
public class HingeItemProvider extends BasisFunctionItemProvider {
	/**
	 * This constructs an instance from a factory and a notifier. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public HingeItemProvider(AdapterFactory adapterFactory) {
		super(adapterFactory);
	}

	/**
	 * This returns the property descriptors for the adapted class. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	@Override
	public List<IItemPropertyDescriptor> getPropertyDescriptors(Object object) {
		if (itemPropertyDescriptors == null) {
			super.getPropertyDescriptors(object);

			addKnotPropertyDescriptor(object);
			addSidePropertyDescriptor(object);
		}
		return itemPropertyDescriptors;
	}

	/**
	 * This adds a property descriptor for the Knot feature.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected void addKnotPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add
			(createItemPropertyDescriptor
				(((ComposeableAdapterFactory)adapterFactory).getRootAdapterFactory(),
				 getResourceLocator(),
				 getString("_UI_Hinge_knot_feature"),
				 getString("_UI_PropertyDescriptor_description", "_UI_Hinge_knot_feature", "_UI_Hinge_type"),
				 SystemPackage.Literals.HINGE__KNOT,
				 true,
				 false,
				 false,
				 ItemPropertyDescriptor.GENERIC_VALUE_IMAGE,
				 null,
				 null));
	}

	/**
	 * This adds a property descriptor for the Side feature.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected void addSidePropertyDescriptor(Object object) {
		itemPropertyDescriptors.add
			(createItemPropertyDescriptor
				(((ComposeableAdapterFactory)adapterFactory).getRootAdapterFactory(),
				 getResourceLocator(),
				 getString("_UI_Hinge_side_feature"),
				 getString("_UI_PropertyDescriptor_description", "_UI_Hinge_side_feature", "_UI_Hinge_type"),
				 SystemPackage.Literals.HINGE__SIDE,
				 true,
				 false,
				 false,
				 ItemPropertyDescriptor.GENERIC_VALUE_IMAGE,
				 null,
				 null));
	}

	/**
	 * This specifies how to implement {@link #getChildren} and is used to deduce an appropriate feature for an
	 * {@link org.eclipse.emf.edit.command.AddCommand}, {@link org.eclipse.emf.edit.command.RemoveCommand} or
	 * {@link org.eclipse.emf.edit.command.MoveCommand} in {@link #createCommand}.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Collection<? extends EStructuralFeature> getChildrenFeatures(Object object) {
		if (childrenFeatures == null) {
			super.getChildrenFeatures(object);
			childrenFeatures.add(SystemPackage.Literals.HINGE__VARIABLE_REFERENCE);
		}
		return childrenFeatures;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected EStructuralFeature getChildFeature(Object object, Object child) {
		// Check the type of the specified child object and return the proper feature to use for
		// adding (see {@link AddCommand}) it as a child.

		return super.getChildFeature(object, child);
	}

	/**
	 * This returns Hinge.gif.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object getImage(Object object) {
		return overlayImage(object, getResourceLocator().getImage("full/obj16/Hinge"));
	}

	/**
	 * This returns the label text for the adapted class. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	@Override
	public String getText(Object object) {
		Hinge hinge = (Hinge) object;
		String postfix = "";
		if (hinge.getCoefficient() != null && hinge.getKnot() != null && hinge.getVariableReference() != null) {
			if (hinge.getVariableReference() instanceof WorkloadIntensity) {
				WorkloadIntensity workloadIntensity = (WorkloadIntensity) hinge.getVariableReference();
				if (workloadIntensity.getInterfaceProvidingRole() != null
						&& workloadIntensity.getInterfaceProvidingRole().getName() != null
						&& workloadIntensity.getSignature() != null
						&& workloadIntensity.getSignature().getName() != null) {
					String variableReferenceName = workloadIntensity.getInterfaceProvidingRole().getName() + "/"
							+ workloadIntensity.getSignature().getName();
					if (hinge.getSide().getValue() == Side.LEFT_VALUE) {
						postfix = " " + getCoefficientString(hinge.getCoefficient()) + " MAX(0 | " + hinge.getKnot()
								+ " - " + variableReferenceName + ")";
					} else {
						postfix = " " + getCoefficientString(hinge.getCoefficient()) + " MAX(0 | "
								+ variableReferenceName + " - " + hinge.getKnot() + ")";
					}
				}
			}
		}

		return getString("_UI_Hinge_type") + postfix;
	}

	private String getCoefficientString(BigDecimal coefficient) {
		if (coefficient.doubleValue() >= 0)
			return "+ " + coefficient.doubleValue();
		return "- " + Math.abs(coefficient.doubleValue());
	}

	/**
	 * This handles model notifications by calling {@link #updateChildren} to update any cached
	 * children and by creating a viewer notification, which it passes to {@link #fireNotifyChanged}.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void notifyChanged(Notification notification) {
		updateChildren(notification);

		switch (notification.getFeatureID(Hinge.class)) {
			case SystemPackage.HINGE__KNOT:
			case SystemPackage.HINGE__SIDE:
				fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), false, true));
				return;
			case SystemPackage.HINGE__VARIABLE_REFERENCE:
				fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), true, false));
				return;
		}
		super.notifyChanged(notification);
	}

	/**
	 * This adds {@link org.eclipse.emf.edit.command.CommandParameter}s describing the children
	 * that can be created under this object.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected void collectNewChildDescriptors(Collection<Object> newChildDescriptors, Object object) {
		super.collectNewChildDescriptors(newChildDescriptors, object);

		newChildDescriptors.add
			(createChildParameter
				(SystemPackage.Literals.HINGE__VARIABLE_REFERENCE,
				 SystemFactory.eINSTANCE.createWorkloadIntensity()));
	}

}
