/**
 * Copyright 2009-2012, Descartes Research Group, SDQ, IPD, KIT
 */
package tools.descartes.dml.mm.applicationlevel.system.provider;


import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.ResourceLocator;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.ViewerNotification;

import tools.descartes.dml.mm.applicationlevel.provider.ApplicationlevelEditPlugin;
import tools.descartes.dml.mm.applicationlevel.repository.AssemblyContext;
import tools.descartes.dml.mm.applicationlevel.repository.InterfaceProvidingRole;
import tools.descartes.dml.mm.applicationlevel.servicebehavior.ServicebehaviorPackage;
import tools.descartes.dml.mm.applicationlevel.servicebehavior.provider.ServiceBehaviorAbstractionItemProvider;
import tools.descartes.dml.mm.applicationlevel.system.SystemPackage;
import tools.descartes.dml.mm.applicationlevel.system.WEKAModel;
import tools.descartes.dml.mm.applicationlevel.util.CustomItemPropertyDescriptor;
import tools.descartes.dml.mm.applicationlevel.util.NavigationCommand;
import tools.descartes.dml.mm.applicationlevel.util.NavigationUtil;

/**
 * This is the item provider adapter for a {@link tools.descartes.dml.mm.applicationlevel.system.WEKAModel} object.
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * @generated
 */
public class WEKAModelItemProvider extends ServiceBehaviorAbstractionItemProvider {
	/**
	 * This constructs an instance from a factory and a notifier.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public WEKAModelItemProvider(AdapterFactory adapterFactory) {
		super(adapterFactory);
	}

	/**
	 * This returns the property descriptors for the adapted class.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public List<IItemPropertyDescriptor> getPropertyDescriptors(Object object) {
		if (itemPropertyDescriptors == null) {
			super.getPropertyDescriptors(object);

			addAbsoluteModelPathPropertyDescriptor(object);
		}
		return itemPropertyDescriptors;
	}

	/**
	 * This adds a property descriptor for the Absolute Model Path feature.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected void addAbsoluteModelPathPropertyDescriptor(Object object) {
		itemPropertyDescriptors.add
			(createItemPropertyDescriptor
				(((ComposeableAdapterFactory)adapterFactory).getRootAdapterFactory(),
				 getResourceLocator(),
				 getString("_UI_WEKAModel_absoluteModelPath_feature"),
				 getString("_UI_PropertyDescriptor_description", "_UI_WEKAModel_absoluteModelPath_feature", "_UI_WEKAModel_type"),
				 SystemPackage.Literals.WEKA_MODEL__ABSOLUTE_MODEL_PATH,
				 true,
				 false,
				 false,
				 ItemPropertyDescriptor.GENERIC_VALUE_IMAGE,
				 null,
				 null));
	}

	/**
	 * This returns WEKAModel.gif.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object getImage(Object object) {
		return overlayImage(object, getResourceLocator().getImage("full/obj16/WEKAModel"));
	}

	/**
	 * This returns the label text for the adapted class.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public String getText(Object object) {
		String label = ((WEKAModel)object).getAbsoluteModelPath();
		return label == null || label.length() == 0 ?
			getString("_UI_WEKAModel_type") :
			getString("_UI_WEKAModel_type") + " " + label;
	}


	/**
	 * This handles model notifications by calling {@link #updateChildren} to update any cached
	 * children and by creating a viewer notification, which it passes to {@link #fireNotifyChanged}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void notifyChanged(Notification notification) {
		updateChildren(notification);

		switch (notification.getFeatureID(WEKAModel.class)) {
			case SystemPackage.WEKA_MODEL__ABSOLUTE_MODEL_PATH:
				fireNotifyChanged(new ViewerNotification(notification, notification.getNotifier(), false, true));
				return;
		}
		super.notifyChanged(notification);
	}

	/**
	 * This adds {@link org.eclipse.emf.edit.command.CommandParameter}s describing the children
	 * that can be created under this object.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected void collectNewChildDescriptors(Collection<Object> newChildDescriptors, Object object) {
		super.collectNewChildDescriptors(newChildDescriptors, object);
	}

	/**
	 * Return the resource locator for this item provider's resources.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public ResourceLocator getResourceLocator() {
		return ApplicationlevelEditPlugin.INSTANCE;
	}

	/**
	 * This adds a property descriptor for the Interface Providing Role feature.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	@Override
	protected void addInterfaceProvidingRolePropertyDescriptor(Object object) {
		NavigationCommand command = new NavigationCommand() {
			@Override
			public Collection<?> navigate(Object object) {
				WEKAModel workloadIntensity = (WEKAModel) object;
				AssemblyContext context = NavigationUtil.getParentAssemblyContext(workloadIntensity);
				if (context.getEncapsulatedComponent() == null)
					return null;

				List<InterfaceProvidingRole> interProv = context.getEncapsulatedComponent()
						.getInterfaceProvidingRoles();
				if (interProv.isEmpty())
					return null;
				return interProv;
			}
		};
		itemPropertyDescriptors.add(
				new CustomItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory).getRootAdapterFactory(),
						getResourceLocator(), getString("_UI_StatisticalModel_interfaceProvidingRole_feature"),
						getString("_UI_PropertyDescriptor_description",
								"_UI_StatisticalModel_interfaceProvidingRole_feature", "_UI_StatisticalModel_type"),
						ServicebehaviorPackage.Literals.SERVICE_BEHAVIOR_ABSTRACTION__INTERFACE_PROVIDING_ROLE, true,
						false, true, null, null, null, command));
	}

	/**
	 * This adds a property descriptor for the Signature feature. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	protected void addSignaturePropertyDescriptor(Object object) {
		NavigationCommand command = new NavigationCommand() {
			@Override
			public Collection<?> navigate(Object object) {
				WEKAModel workloadIntensity = (WEKAModel) object;
				if (workloadIntensity.getInterfaceProvidingRole() == null
						|| workloadIntensity.getInterfaceProvidingRole().getInterface() == null)
					return null;
				return workloadIntensity.getInterfaceProvidingRole().getInterface().getSignatures();
			}
		};
		itemPropertyDescriptors.add(
				new CustomItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory).getRootAdapterFactory(),
						getResourceLocator(), getString("_UI_StatisticalModel_signature_feature"),
						getString("_UI_PropertyDescriptor_description", "_UI_StatisticalModel_signature_feature",
								"_UI_StatisticalModel_type"),
						ServicebehaviorPackage.Literals.SERVICE_BEHAVIOR_ABSTRACTION__DESCRIBED_SIGNATURE, true, false,
						true, null, null, null, command));
	}
}
