/**
 * Copyright 2009-2012, Descartes Research Group, SDQ, IPD, KIT
 */
package tools.descartes.dml.mm.applicationlevel.system.provider;


import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.ResourceLocator;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;

import tools.descartes.dml.identifier.bugfix.BugfixCDOItemProviderAdapter;
import tools.descartes.dml.mm.applicationlevel.provider.ApplicationlevelEditPlugin;
import tools.descartes.dml.mm.applicationlevel.repository.AssemblyContext;
import tools.descartes.dml.mm.applicationlevel.repository.InterfaceProvidingRole;
import tools.descartes.dml.mm.applicationlevel.system.SystemPackage;
import tools.descartes.dml.mm.applicationlevel.system.WorkloadIntensity;
import tools.descartes.dml.mm.applicationlevel.util.CustomItemPropertyDescriptor;
import tools.descartes.dml.mm.applicationlevel.util.NavigationCommand;
import tools.descartes.dml.mm.applicationlevel.util.NavigationUtil;

/**
 * This is the item provider adapter for a {@link tools.descartes.dml.mm.applicationlevel.system.WorkloadIntensity} object.
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * @generated
 */
public class WorkloadIntensityItemProvider 
	extends BugfixCDOItemProviderAdapter
	implements
		IEditingDomainItemProvider,
		IStructuredItemContentProvider,
		ITreeItemContentProvider,
		IItemLabelProvider,
		IItemPropertySource {
	/**
	 * This constructs an instance from a factory and a notifier.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public WorkloadIntensityItemProvider(AdapterFactory adapterFactory) {
		super(adapterFactory);
	}

	/**
	 * This returns the property descriptors for the adapted class.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public List<IItemPropertyDescriptor> getPropertyDescriptors(Object object) {
		if (itemPropertyDescriptors == null) {
			super.getPropertyDescriptors(object);

			addInterfaceProvidingRolePropertyDescriptor(object);
			addSignaturePropertyDescriptor(object);
		}
		return itemPropertyDescriptors;
	}

	/**
	 * This adds a property descriptor for the Interface Providing Role feature.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	protected void addInterfaceProvidingRolePropertyDescriptor(Object object) {
		NavigationCommand command = new NavigationCommand() {
			@Override
			public Collection<?> navigate(Object object) {
				WorkloadIntensity workloadIntensity = (WorkloadIntensity) object;
				AssemblyContext context = NavigationUtil.getParentAssemblyContext(workloadIntensity);
				if (context.getEncapsulatedComponent() == null)
					return null;

				List<InterfaceProvidingRole> interProv = context.getEncapsulatedComponent()
						.getInterfaceProvidingRoles();
				if (interProv.isEmpty())
					return null;
				return interProv;
			}
		};
		itemPropertyDescriptors.add(
				new CustomItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory).getRootAdapterFactory(),
						getResourceLocator(), getString("_UI_WorkloadIntensity_interfaceProvidingRole_feature"),
						getString("_UI_PropertyDescriptor_description",
								"_UI_WorkloadIntensity_interfaceProvidingRole_feature", "_UI_WorkloadIntensity_type"),
						SystemPackage.Literals.WORKLOAD_INTENSITY__INTERFACE_PROVIDING_ROLE, true, false, true, null,
						null, null, command));
	}

	/**
	 * This adds a property descriptor for the Signature feature. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	protected void addSignaturePropertyDescriptor(Object object) {
		NavigationCommand command = new NavigationCommand() {
			@Override
			public Collection<?> navigate(Object object) {
				WorkloadIntensity workloadIntensity = (WorkloadIntensity) object;
				if (workloadIntensity.getInterfaceProvidingRole() == null
						|| workloadIntensity.getInterfaceProvidingRole().getInterface() == null)
					return null;
				return workloadIntensity.getInterfaceProvidingRole().getInterface().getSignatures();
			}
		};
		itemPropertyDescriptors.add(
				new CustomItemPropertyDescriptor(((ComposeableAdapterFactory) adapterFactory).getRootAdapterFactory(),
				 getResourceLocator(),
				 getString("_UI_WorkloadIntensity_signature_feature"),
				 getString("_UI_PropertyDescriptor_description", "_UI_WorkloadIntensity_signature_feature", "_UI_WorkloadIntensity_type"),
				 SystemPackage.Literals.WORKLOAD_INTENSITY__SIGNATURE,
				 true,
				 false,
				 true,
				 null,
				 null,
						null, command));
	}

	/**
	 * This returns WorkloadIntensity.gif.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object getImage(Object object) {
		return overlayImage(object, getResourceLocator().getImage("full/obj16/WorkloadIntensity"));
	}

	/**
	 * This returns the label text for the adapted class. <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	@Override
	public String getText(Object object) {
		WorkloadIntensity workloadIntensity = (WorkloadIntensity) object;
		String postfix = "";
		if (workloadIntensity.getInterfaceProvidingRole() != null
				&& workloadIntensity.getInterfaceProvidingRole().getName() != null
				&& workloadIntensity.getSignature() != null && workloadIntensity.getSignature().getName() != null)
			postfix = " " + workloadIntensity.getInterfaceProvidingRole().getName() + "/"
					+ workloadIntensity.getSignature().getName();
		return getString("_UI_WorkloadIntensity_type") + postfix;
	}


	/**
	 * This handles model notifications by calling {@link #updateChildren} to update any cached
	 * children and by creating a viewer notification, which it passes to {@link #fireNotifyChanged}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public void notifyChanged(Notification notification) {
		updateChildren(notification);
		super.notifyChanged(notification);
	}

	/**
	 * This adds {@link org.eclipse.emf.edit.command.CommandParameter}s describing the children
	 * that can be created under this object.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	protected void collectNewChildDescriptors(Collection<Object> newChildDescriptors, Object object) {
		super.collectNewChildDescriptors(newChildDescriptors, object);
	}

	/**
	 * Return the resource locator for this item provider's resources.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public ResourceLocator getResourceLocator() {
		return ApplicationlevelEditPlugin.INSTANCE;
	}

}
